'' =================================================================================================
''
''   File....... jm_mcp4728.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen 
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com 
''   Started.... 
''   Updated....  
''
'' =================================================================================================


con

  MCP4728  = %1100_000_0

  GEN_CALL = $00
  RST      = $06                                                ' reset (DACs updated)
  UPD_ALL  = $08                                                ' update all DACs
  WAKE     = $09                                                ' wake from power-down
  RD_ADDR  = $0C
  WR_SEQ   = $30  
  WR_MULTI = $40
  WR_SNGL  = $58   
  WR_VREF  = $80 
  WR_PWRDN = $A0 
  WR_GAIN  = $C0


  VR_VDD   = %0_00_0                                            ' configuration bits (write)
  VR_2048  = %1_00_0

  PD_NORM  = %0_00_0
  PD_1K    = %0_01_0
  PD_100K  = %0_10_0
  PD_500K  = %0_11_0

  GAIN_1X  = %0_00_0
  GAIN_2X  = %0_00_1 


obj

  i2c : "jm_i2c_basic"                                          ' use boot EE buss

  
var

  long  slaveid 


pub start(device)

'' Start MCP4728 device; setup slave ID for this object

  i2c.setup                                                     ' connect to I2C buss
  slaveid := MCP4728 | ((device & %111) << 1)                   ' setup slave ID byte


pub reset

'' General call reset -- loads DAC registers

  i2c.start
  i2c.write(GEN_CALL)
  i2c.write(RST)                                                ' reset device
  i2c.stop


pub update

'' Updates all DAC outputs

  i2c.start
  i2c.write(GEN_CALL)
  i2c.write(UPD_ALL)                                            ' update dacs
  i2c.stop

  
pub wake_up

'' Resets power-down bits

  i2c.start
  i2c.write(GEN_CALL)
  i2c.write(WAKE_UP)                                            ' wake from power-down
  i2c.stop


pub write(ch, cfg, value, wrhold) | status

'' Writes single channel and updates EEPROM with value
'' -- section 5.6.4 of documentation
'' -- cfg bits = Vref (MSB), PD1, PD0, gain (LSB)
'' -- value is 12-bit output (0 to 4095)
'' -- if wrhold == true, wait for EEPROM write to finish

  if ((ch => 0) and (ch =< 3))                                  ' legal channel
    value := 0 #> value <# $FFF                                 '  limit to legal range
    i2c.start                                                  
    i2c.write(slaveid)
    i2c.write(WR_SNGL | (ch << 1))                              '  command with ch
    i2c.write(((cfg & $0F) << 4) | (value.byte[1] & $0F))       '  send cfg bits and value to channel 
    i2c.write(value.byte[0])
    i2c.stop

    ' let eeprom write finish before returning
    ' -- testing = ~30.5ms to complete write cycle

    if (wrhold)                                                 ' if write hold specified
      repeat
        i2c.start
        i2c.write(slaveid | %1)                                 '  read mode
        status := i2c.read(i2c#NAK)                             '  read status
        i2c.stop                                                 
        if (status & $80)                                       '  check for RDY\nBUSY == 1
          quit

    return value

  else
    return -1


pub read(ch) | value

'' Reads 12-bit DAC input register for channel
'' -- figure 5-15 in documentation 

  value := 0

  if ((ch => 0) and (ch =< 3))                                  ' legal channel
    i2c.start                                                  
    i2c.write(slaveid | %1)                                     ' read
    repeat ((ch*6) + 1)                                         ' skip over extra bytes   
      value := i2c.read(i2c#ACK)
    value.byte[1] := i2c.read(i2c#ACK)                          ' read value
    value.byte[0] := i2c.read(i2c#ACK)
    i2c.stop
    return value & $FFF                                         ' truncate to 12 bits

  else
    return -1


pub set_vref(vrefbits)

'' Set Vref for all channels
'' -- 0 bit = Vdd, 1 bit = internal (2.048v)
'' -- channel bits: %ABCD

  i2c.start
  i2c.write(slaveid)
  i2c.write(WR_VREF | (vrefbits & $0F))
  i2c.stop
    

pub set_gain(gainbits)

'' Set gain for all channels
'' -- 0 bit = 1x, 1 bit = 2x
'' -- works only with internal Vref
'' -- channel bits: %ABCD 

  i2c.start
  i2c.write(slaveid)
  i2c.write(WR_GAIN | (gainbits & $0F))
  i2c.stop

  
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}